#!/usr/bin/env python

"""
Date processing functions.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import date, datetime, timedelta
from pytz import timezone, UnknownTimeZoneError
import re

# iCalendar date and datetime parsing (from DateSupport in MoinSupport).

date_icalendar_regexp_str = ur'(?P<year>[0-9]{4})(?P<month>[0-9]{2})(?P<day>[0-9]{2})'
datetime_icalendar_regexp_str = date_icalendar_regexp_str + \
    ur'(?:' \
    ur'T(?P<hour>[0-2][0-9])(?P<minute>[0-5][0-9])(?P<second>[0-6][0-9])' \
    ur'(?P<utc>Z)?' \
    ur')?'

match_date_icalendar = re.compile(date_icalendar_regexp_str, re.UNICODE).match
match_datetime_icalendar = re.compile(datetime_icalendar_regexp_str, re.UNICODE).match

def to_utc_datetime(dt):
    if not dt:
        return None
    elif isinstance(dt, datetime):
        return to_timezone(dt, "UTC")
    else:
        return dt

def to_timezone(dt, name):
    try:
        tz = name and timezone(name) or None
    except UnknownTimeZoneError:
        tz = None
    return to_tz(dt, tz)

def to_tz(dt, tz):
    if tz is not None and isinstance(dt, datetime):
        if not dt.tzinfo:
            return tz.localize(dt)
        else:
            return dt.astimezone(tz)
    else:
        return dt

def format_datetime(dt):
    if not dt:
        return None
    elif isinstance(dt, datetime):
        if dt.tzname() == "UTC":
            return dt.strftime("%Y%m%dT%H%M%SZ")
        else:
            return dt.strftime("%Y%m%dT%H%M%S")
    else:
        return dt.strftime("%Y%m%d")

def get_datetime(value, attr=None):

    """
    Return a datetime object from the given 'value' in iCalendar format, using
    the 'attr' mapping (if specified) to control the conversion.
    """

    if not attr or attr.get("VALUE") in (None, "DATE-TIME"):
        m = match_datetime_icalendar(value)
        if m:
            year, month, day, hour, minute, second = map(m.group, [
                "year", "month", "day", "hour", "minute", "second"
                ])

            if hour and minute and second:
                dt = datetime(
                    int(year), int(month), int(day), int(hour), int(minute), int(second)
                    )

                # Impose the indicated timezone.
                # NOTE: This needs an ambiguity policy for DST changes.

                return to_timezone(dt, m.group("utc") and "UTC" or attr and attr.get("TZID") or None)

    if not attr or attr.get("VALUE") == "DATE":
        m = match_date_icalendar(value)
        if m:
            year, month, day = map(m.group, ["year", "month", "day"])
            return date(int(year), int(month), int(day))

    return None

def get_start_of_day(dt, tzid=None):
    return datetime(dt.year, dt.month, dt.day, 0, 0, tzinfo=(tzid and timezone(tzid) or dt.tzinfo))

def get_end_of_day(dt, tzid=None):
    return get_start_of_day(dt + timedelta(1), tzid)

def ends_on_same_day(dt, end):
    return (
        dt.date() == end.date() or
        end == get_end_of_day(dt)
        )

def get_timestamp():
    return format_datetime(to_timezone(datetime.utcnow(), "UTC"))

# vim: tabstop=4 expandtab shiftwidth=4
